# -*- coding: utf-8 -*-

import json
from contextlib import closing

import unicodecsv


try:
	import cStringIO as StringIO
except ImportError:
	import StringIO


def json_to_csv(jsonData, fieldnames=None):
	"""
	Returns a CSV string from the given JSON data string.

	>>> import json
	>>> data = [
	... {"col1": "foo", "col2": "Ȓȳǡɴ"},
	... {"col1": "foo", "col2": "bar"},
	... {"col1": "foo", "col2": "bar"},
	... ]
	>>> csvData = json_to_csv(json.dumps(data))
	"""
	d = json.loads(jsonData)
	fieldnames = fieldnames or d[0].keys()
	return dict_to_csv(d, fieldnames)

def dict_to_csv(d, fieldnames):
	"""
	Returns a CSV string from the given list of dictionaries.

	This assumes that each dictionary is simple and each contains the same keys.

	>>> import json
	>>> data = [
	... {"col1": "foo", "col2": "Ȓȳǡɴ"},
	... {"col1": "foo", "col2": "bar"},
	... {"col1": "foo", "col2": "bar"},
	... ]
	>>> csvData = dict_to_csv(data, fieldnames=["col1", "col2"])
	"""
	buf = None
	with closing(StringIO.StringIO()) as f:
		dictWriter = unicodecsv.DictWriter(f, fieldnames)
		dictWriter.writeheader()
		dictWriter.writerows(d)

		buf = f.getvalue()

	return buf



from django.contrib import admin
from django.core import serializers
from django.http import HttpResponse

from .utils import json_to_csv


def serialize_queryset(queryset, format):
	data = serializers.serialize(format, queryset)
	return data

def export(queryset, format):
	if format == "csv":
		# TODO: Less hacky
		import json
		data = serialize_queryset(queryset, "json")
		data = json.loads(data)
		dataFlattened = []
		for item in data:
			flattednedItem = item["fields"]
			flattednedItem["pk"] = item["pk"]
			flattednedItem["model"] = item["model"]
			dataFlattened.append(flattednedItem)
		data = json.dumps(dataFlattened)
		data = json_to_csv(data)
	else:
		data = serialize_queryset(queryset, format)

	response = HttpResponse(data, mimetype="application/x-download")
	response["Content-Disposition"] = "attachment;filename=export.{extention}".format(extention=format.lower())

	return response

def export_as_csv_action(modeladmin, request, queryset):
	return export(queryset, format="csv")
export_as_csv_action.short_description = "Export selected items to CSV"

def export_as_json_action(modeladmin, request, queryset):
	return export(queryset, format="json")
export_as_json_action.short_description = "Export selected items to JSON"

def export_as_xml_action(modeladmin, request, queryset):
	return export(queryset, format="xml")
export_as_xml_action.short_description = "Export selected items to XML"

def export_as_yaml_action(modeladmin, request, queryset):
	return export(queryset, format="yaml")
export_as_yaml_action.short_description = "Export selected items to YAML"



# -*- coding: utf-8 -*-
import os

from setuptools import setup, find_packages


README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
	name='django-admin-exporter',
	version='0.2.1',
	url='https://github.com/RyanBalfanz/django-admin-exporter',
	author='Ryan Balfanz',
	author_email='ryan@ryanbalfanz.net',
	description='Simple admin actions to download/export selected items in CSV, JSON, XML, etc.',
	long_description=README,
	packages = find_packages(exclude=("example_project",)),
	zip_safe=False,
	include_package_data=True,
	platforms='any',
	install_requires=['unicodecsv'],
	classifiers=[
		'Environment :: Web Environment',
		'Intended Audience :: Developers',
		'Operating System :: OS Independent',
		'Programming Language :: Python',
		'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
		'Topic :: Software Development :: Libraries :: Python Modules'
	]
)
